"""Data.py--this file contains user input data for multiscale multiBreath run"""

reuse3D=False # True only if 1D code called with reused 3D data
dtio=0.01     # I/O timestep for data file exchange between 3D and 1D
nbreaths=3    # Number of full breaths to be simulated

# Explicitly list inhale/exhale time intervals
# and inhale masses for joint 3D/1D computation
inhales=[ [0.00, 1.55], [ 2.98,  4.53], [ 5.96,  7.51]]
exhales=[ [1.55, 2.98], [ 4.53,  5.96], [ 7.51,  8.94]] 
inhaleMasses=[8.e-8, 8.e-8, 8.e-8]   #MKS units

tstart=0.00   #Start time of simulation
tend=8.94     #End time of simulation

#Subtree FRC Volumes in milliliters. (This is only quantity not in MKS.)
VolFRCinMl=[ 429.12, 307.83, 279.19, 162.43, 318.61, 442.65, 276.81, 258.39, 238.74, 163.41, 268.49]

particleDiameter=2.9e-6 #Particle diameter in MKS
particleDensity=1000.   #Particle density in MKS

params1D={'segmentsPerAirway': 20,  #used by MPPD
        'MixingModel': 1}           #Alveolar dispersion 1=on; 2=off

params3D={'inhaleParcelsPerSecond': 1.e+4, #3D injection rate for parcels
        'maxCoParticle': 0.4,              #3D max Courant number for particle tracking
        'maxCoFlow': 7.5,                  #3D max Courant number for flow computation
        'maxDeltaT': 3.e-5}                #3D maximum timestep

output3Dfile='Lung.out'  #Slurm output file--important to keep and not edit
error3Dfile='Lung.err'   #Slurm input file

ngen=24  #Total number of 3D+1D airflow generations

#name3D contains string names of interface exit surfaces used by 3D code
name3D=['outRB5', 'outLB4', 'outLB1', 'outRB6', 'outRB3', 'outRB2', 
        'outRB1', 'outLB5', 'outLB3', 'outRB4', 'outLB2']

#name1D contains corresponding string names of subtree inlets used by MPPD code
name1D=['outlet1', 'outlet2', 'outlet3', 'outlet4', 'outlet5', 'outlet6', 
        'outlet7', 'outlet8', 'outlet9', 'outlet10', 'outlet11']

#depPatch contains string names of 3D wall deposition surface(s).
depPatch=['lungWalls'] 

################# User Data above this line ######################


params1D['FRCinMl']=VolFRCinMl
shortname1D=[x[6:] for x in name1D]
nDepPatch=len(depPatch)
ndist=len(name3D)

#Long and shortened string names for injectors (associated with 3D/1D interface)
#Appended to the end of these lists is an entry for the tracheal injector
injectors=[f'outletInjection_{x}' for x in name3D]+['inletInjection_inlet']
shortinjectors=[x for x in name3D]+['inlet']

# Add auto-generated time intervals and masses for
# additional 1D-only computation if desired.
if reuse3D:
    last3Dbreath=len(exhales)-1
    period=exhales[last3Dbreath][1]-inhales[last3Dbreath][0]
    for i in range(last3Dbreath+1,nbreaths):
        inhales+=[[inhales[i-1][0]+period,inhales[i-1][1]+period]]
        exhales+=[[exhales[i-1][0]+period,exhales[i-1][1]+period]]
        inhaleMasses+=[inhaleMasses[last3Dbreath]]

