#!/bin/env python3
"""multiBreath.py--- 3D/1D hybrid breathing deposition simulation main script

Code currently uses FOAM routine MPPICFoam for 3D computation,
MPPD for 1D computation, and this script must be called with slurm"""

numcores=12  #This number should match ntasks-per-node*N in slurm data below

#SBATCH --open-mode=append
##SBATCH -A biocfd
##SBATCH -p slurm
#SBATCH -t 7-00:00
#SBATCH -N 1
#SBATCH --ntasks-per-node=12
#SBATCH -J multiBreath
#SBATCH --mail-type=END
#SBATCH --exclusive
#SBATCH --error=Lung.err
#SBATCH --output=Lung.out
#SBATCH --mail-user=andrew.kuprat@pnnl.gov

import sys
sys.path.append(".")

import os
import simclass
import pre3D, post3D
import call1D
from data import reuse3D

sim=simclass.SimClass()

print(f'\n\n\n>>>>>>>>>>>>>>>>>>>>>>>>>>>\n>>>>>>>>>>>>>>>>>>>>>>>>>>>\n')
print(f'BREATHJOB submitted with tstart={sim.tstart}, tend={sim.tend}\n')
print(f'params1D={sim.params1D}\n')
print(f'params3D={sim.params3D}\n')
print(f'particleDiameter={sim.particleDiameter}\n')
print(f'particleDensity={sim.particleDensity}\n')
print(f'inhales={sim.inhales}\n')
print(f'exhales={sim.exhales}\n')
print(f'inhaleMasses={sim.inhaleMasses}\n')
print(f'reuse3D={reuse3D}\n')
print(f'>>>>>>>>>>>>>>>>>>>>>>>>>>>\n>>>>>>>>>>>>>>>>>>>>>>>>>>>\n\n')
sys.stdout.flush()

TOL=1.e-8
if reuse3D:
    #In this case call MPPD code using copied 3D data from last 3D-computed breath.
    clonedbreath=sim.currbreath
    for breath in range(clonedbreath+1,len(sim.inhales)):
        sim.currbreath=breath
        sim.clone3Ddata(clonedbreath)
        call1D.call1Donly(sim)

else:    
    #FOAMSPECIFIC--- Hand-off occurs between 3D FOAM (MPPICFoam) and 1D MPPD code
    init=True
    startbreath=sim.currbreath
    for breath in range(startbreath,len(sim.inhales)):
        if (not init) or sim.tstart+TOL<sim.inhales[breath][1]:
            pre3D.pre3Dinhale(sim)  # uses sim to prepare 3D input files
            com=f'source appenv.sh ;mpirun -np {numcores} MPPICFoam -parallel'
            if os.system(com):
                sys.exit(f'>>> error processing {com}\n')
            else:    
                print(f'>>>> Executed MPPICFoam for Inhale\n',flush=True)
        if (not init) or sim.tstart-TOL<=sim.inhales[breath][1]:
            post3D.post3Dinhale(sim) # also call1D and updates and writes out sim 

        if sim.tend-TOL<=sim.inhales[breath][1]:
            break

        if (not init) or sim.tstart+TOL<sim.exhales[breath][1]:
            pre3D.pre3Dexhale(sim) # uses sim to prepare 3D input files
            com=f'source appenv.sh ;mpirun -np {numcores} MPPICFoam -parallel'
            if os.system(com):
                sys.exit(f'>>> error processing {com}\n')
            else:    
                print(f'>>>> Executed MPPICFoam for Exhale\n',flush=True)
        if (not init) or sim.tstart-TOL<=sim.exhales[breath][1]:  #should always be true
            post3D.post3Dexhale(sim)# updates and writes out sim

        if sim.tend-TOL<=sim.exhales[breath][1]:
            break

        sim.currbreath+=1
        init=False

