# custom user data for 11 subtree case
# can be commented out
SubtreeClusters=[[3,9,11],[2,8],[5,6,7],[1,4,10]]
SubtreeClusterNames=['LB123','LB45','RB123','RB456']
GenClusters=[[4,5,6,7,8,9,10],[11,12,13,14,15,16],[17,18],[19],[20],[21],[22,23]]
GenClusterNames=['Gen4-10','Gen11-16','Gen17-18','Gen19','Gen20','Gen21','Gen22-23']
SubtreeGenCoClusters=[ [[3,9,11],[20]], [[2,8],[20]] ]
SubtreeGenCoClusterNames=['LB123/Gen20','LB45/Gen20']

################# User Data above this line ######################

import sys, getopt
import os
import yaml
from simclass import SimClass
import xlsxwriter
import data
import numpy as np

# useful global variables set in main()
numMultiTimePartial=0
numMultiTimeCompleted=0

def main(argv):
    # Default filename and scale
    directoryname=os.path.basename(os.getcwd())
    filename=f'results_{directoryname}.xlsx'
    scale=1.

    try:
        opts, args = getopt.getopt(argv,"s:o:")
    except getopt.GetoptError:
        print ('usage: genexcelrs.py -s <unit scale in kg> -o <outputfile>')
        sys.exit(2)
    for opt, arg in opts:
        if opt=='-s':
            scale=float(arg)
        elif opt=='-o':
            filename=arg
            if filename[-5:] != '.xlsx':
                filename+='.xlsx'
    print(f'We have got scale={scale}')
    print(f'We have got filename={filename}')

    workbook=xlsxwriter.Workbook(filename, {'nan_inf_to_errors': True})
    bold=workbook.add_format({'bold': 1})

    sheetname='summary'
    summary=workbook.add_worksheet(sheetname)
    # val_format=workbook_add_format({'num_format': '????'})

    sheetnamepb='per breath'
    perbreath=workbook.add_worksheet(sheetnamepb)

    fname='sim.yaml'
    with open(fname,'r') as f:
        sim=yaml.unsafe_load(f)

    # Divide kg mass elements in sim by scale
    if scale!=1.:
        sim.massInhaled/=scale
        sim.mass3Ddeposited/=scale
        sim.mass3Dsuspended/=scale
        sim.massExhaled/=scale
        sim.mass1Dto3D/=scale
        sim.mass3Dto1D/=scale
        sim.mass1Ddeposited/=scale
        sim.mass1Dsuspended/=scale

#Default values if SubtreeClusters, GenClusters, SubtreeGenCoClusters not defined
    if not 'SubtreeClusters' in globals():
        global SubtreeClusters, SubtreeClusterNames
        SubtreeClusters=[[ind+1 for ind,_ in enumerate(data.name1D)]] #Outlet numbering starts from 1
        SubtreeClusterNames=[f'Subtrees 1-{len(SubtreeClusters[0])}']
        defaultSubtreeClusters=True
    else:
        defaultSubtreeClusters=False
    if not 'GenClusters' in globals():
        global GenClusters, GenClusterNames
        if defaultSubtreeClusters:
            GenClusters=[]
            GenClusterNames=[]
        else:
            GenClusters=[[ind for ind in range(sim.ngen)]] #Generation numbering starts with zero.
            GenClusterNames=['Sum all 1D Gens']
    if not 'SubtreeGenCoClusters' in globals():
        global SubtreeGenCoClusters, SubtreeGenCoClusterNames
        SubtreeGenCoClusters=[]
        SubtreeGenCoClusterNames=[]
    
    global numMultiTimePartial, numMultiTimeCompleted 
    numMultiTimePartial=round(sim.multiTimePartial/sim.dtio)+1
    numMultiTimeCompleted=round(sim.multiTimeCompleted/sim.dtio)+1

    sum3Dto1D,sum1Dto3D=write_transfers(workbook,sim)

    sum1Ddep=write_1Ddeposition(workbook,sim)
    sum1Dsusp=write_1Dsuspended(workbook,sim)

    if len(GenClusters):
        write_1Ddep_by_gen(workbook,sim)
        write_1Dsusp_by_gen(workbook,sim)

    if len(SubtreeGenCoClusters):
        write_1Ddep_by_subtree_and_gen(workbook,sim)
        write_1Dsusp_by_subtree_and_gen(workbook,sim)

    ioff=1
    altclr=['brown','green','magenta','navy','orange','yellow','silver'] 
    bpchart = workbook.add_chart({'type': 'scatter', 'subtype': 'straight_with_markers'})
    CFDchart = workbook.add_chart({'type': 'scatter', 'subtype': 'straight_with_markers'})
    nextcol=0
#Write Time 
    summary.write(0,nextcol, 'Time', bold) 
    for i in range(numMultiTimePartial):
        summary.write(ioff+i,nextcol,sim.time[i])
    nextcol+=1
#Write Particle Mass Inhaled
    summary.write(0,nextcol, 'Inhale model (3D)', bold)
    for i in range(numMultiTimePartial):
        summary.write(ioff+i,nextcol,sim.massInhaled[i])
    ilast=ioff+numMultiTimePartial-1    
    bpchart.add_series({
        'name': 'inhale model (3D)',
        'categories': [sheetname,ioff, 0,ilast, 0],
        'values': [sheetname,ioff,nextcol,ilast,nextcol],
        'line': {'color': 'blue'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'blue'},
            'fill':   {'color': 'blue'},
            },
        })
    nextcol+=1
#Write Patch Deposited 3D Masses
    if sim.nDepPatch>1:
        for ind,patch in enumerate(data.depPatch):
            summary.write(0,nextcol, f'{patch} 3D', bold)
            ilast=ioff+numMultiTimeCompleted-1
            for i in range(numMultiTimeCompleted):
                summary.write(ioff+i,nextcol,sim.mass3Ddeposited[ind,i])
            CFDchart.add_series({
                'name': f'{patch} 3D',
                'categories': [sheetname,ioff,0,ilast,0],
                'values': [sheetname,ioff,nextcol,ilast,nextcol],
                'line': {'color': altclr[ind]},
                'marker': {
                    'type': 'square',
                    'size': 2,
                    'border': {'color': altclr[ind]},
                    'fill':   {'color': altclr[ind]},
                    },
                })
            nextcol+=1
#Write Total Deposited 3D Masses.
    totMass3Ddeposited=np.sum(sim.mass3Ddeposited,axis=0)
    summary.write(0,nextcol, 'TotalDep 3D', bold)
    for i in range(numMultiTimeCompleted):
        summary.write(ioff+i,nextcol,totMass3Ddeposited[i])
    ilast=ioff+numMultiTimeCompleted-1
    CFDchart.add_series({
        'name': 'TotalDep 3D',
        'categories': [sheetname,ioff,0,ilast,0],
        'values': [sheetname,ioff,nextcol,ilast,nextcol],
        'line': {'color': 'red'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'red'},
            'fill':   {'color': 'red'},
            },
        })
    bpchart.add_series({
        'name': 'TotalDep 3D',
        'categories': [sheetname,ioff,0,ilast,0],
        'values': [sheetname,ioff,nextcol,ilast,nextcol],
        'line': {'color': 'red'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'red'},
            'fill':   {'color': 'red'},
            },
        })
    nextcol+=1
#Write Suspended 3D Mass
    summary.write(0,nextcol, 'Suspended 3D', bold)
    for i in range(numMultiTimeCompleted):
        summary.write(ioff+i,nextcol,sim.mass3Dsuspended[i])
    ilast=ioff+numMultiTimeCompleted-1
    CFDchart.add_series({
        'name': 'Suspended 3D',
        'categories': [sheetname,ioff, 0,ilast, 0],
        'values': [sheetname,ioff, nextcol,ilast, nextcol],
        'line': {'color': 'gray'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'gray'},
            'fill':   {'color': 'gray'},
            },
        })
    bpchart.add_series({
        'name': 'Suspended 3D',
        'categories': [sheetname,ioff, 0,ilast, 0],
        'values': [sheetname,ioff, nextcol,ilast, nextcol],
        'line': {'color': 'gray'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'gray'},
            'fill':   {'color': 'gray'},
            },
        })
    nextcol=nextcol+1    
#Write 3D->1D cumulative particle mass transferred
    summary.write(0,nextcol, '3D->1D', bold)
    for i in range(numMultiTimePartial):
        summary.write(ioff+i,nextcol,sum3Dto1D[i])
    ilast=ioff+numMultiTimePartial-1
    bpchart.add_series({
        'name': '3D->1D',
        'categories': [sheetname,ioff, 0,ilast, 0],
        'values': [sheetname,ioff, nextcol,ilast, nextcol],
        'line': {'color': 'lime'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'lime'},
            'fill':   {'color': 'lime'},
            },
        })
    nextcol=nextcol+1    
#Write 1D->3D cumulative particle mass transferred
    summary.write(0,nextcol, '1D->3D', bold)
    for i in range(numMultiTimePartial):
        summary.write(ioff+i,nextcol,sum1Dto3D[i])
    ilast=ioff+numMultiTimePartial-1
    bpchart.add_series({
        'name': '1D->3D',
        'categories': [sheetname,ioff, 0,ilast, 0],
        'values': [sheetname,ioff, nextcol,ilast, nextcol],
        'line': {'color': 'cyan'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'cyan'},
            'fill':   {'color': 'cyan'},
            },
        })
    nextcol=nextcol+1    
#lennet1Dmass=min(len(sim.mass3Dto1D),len(sim.mass1Dto3D))
#net1Dmass=sim.mass3Dto1D[:lennet1Dmass]-sim.mass1Dto3D[:lennet1Dmass]
#Write net mass residing in 1D system
#for i,val in enumerate(net1Dmass):
    net1Dmass=sum3Dto1D[:numMultiTimePartial]-sum1Dto3D[:numMultiTimePartial]
    summary.write(0,nextcol, '1D system mass', bold)
    for i in range(numMultiTimePartial):
        summary.write(ioff+i,nextcol,net1Dmass[i])
    nextcol=nextcol+1    
#Write mass deposited in 1D airways
    summary.write(0,nextcol, 'Deposited 1D', bold)
    for i in range(numMultiTimePartial):
        summary.write(ioff+i,nextcol,sum1Ddep[i])
    ilast=ioff+numMultiTimePartial-1
    bpchart.add_series({
        'name': 'Deposited 1D',
        'categories': [sheetname,ioff, 0,ilast, 0],
        'values': [sheetname,ioff, nextcol,ilast, nextcol],
        'line': {'color': 'black'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'black'},
            'fill':   {'color': 'black'},
            },
        })
    nextcol=nextcol+1    
#Write mass suspended in 1D airways
    summary.write(0,nextcol, 'Suspended 1D', bold)
    for i in range(numMultiTimePartial):
        summary.write(ioff+i,nextcol,sum1Dsusp[i])
    ilast=ioff+numMultiTimePartial-1
    bpchart.add_series({
        'name': 'Suspended 1D',
        'categories': [sheetname,ioff, 0,ilast, 0],
        'values': [sheetname,ioff,nextcol,ilast,nextcol],
        'line': {'color': 'pink'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'pink'},
            'fill':   {'color': 'pink'},
            },
        })
    nextcol=nextcol+1    
#Write mass exhaled from whole system
    summary.write(0,nextcol, 'Exhale Model (3D)', bold)
    for i in range(numMultiTimeCompleted):
        summary.write(ioff+i,nextcol,sim.massExhaled[i])
    ilast=ioff+numMultiTimeCompleted-1
    bpchart.add_series({
        'name': 'Exhale model (3D)',
        'categories': [sheetname,ioff, 0,ilast, 0],
        'values': [sheetname,ioff,nextcol,ilast,nextcol],
        'line': {'color': 'purple'},
        'marker': {
            'type': 'square',
            'size': 2,
            'border': {'color': 'purple'},
            'fill':   {'color': 'purple'},
            },
        })
    nextcol=nextcol+1    

    summary.insert_chart('J7', bpchart)

    summary.insert_chart('A7', CFDchart)

    write_per_breath(perbreath,sim,sum3Dto1D,sum1Dto3D)

    workbook.close()

def write_transfers(workbook,sim):
    sum3Dto1D=np.zeros(numMultiTimePartial)
    sum1Dto3D=np.zeros(numMultiTimePartial)

    sh_3Dto1D=workbook.add_worksheet('3D->1D')
    sh_1Dto3D=workbook.add_worksheet('1D->3D')
    sh_3Dto1D.write(0,0,'Time')
    sh_3Dto1D.write(0,1,'Label')
    sh_1Dto3D.write(0,0,'Time')
    sh_1Dto3D.write(0,1,'Label')
    joff=2
    for j,_ in enumerate(data.name3D):
        sh_3Dto1D.write(0,j+joff,data.name3D[j])
        sh_3Dto1D.write(1,j+joff,data.name1D[j])
        sh_1Dto3D.write(0,j+joff,data.name3D[j])
        sh_1Dto3D.write(1,j+joff,data.name1D[j])
    sumcol=len(data.name3D)+joff+2
    sh_3Dto1D.write(0,sumcol,'sum')
    sh_1Dto3D.write(0,sumcol,'sum')
    ioff=2
    for i in range(numMultiTimePartial):
        sh_1Dto3D.write(i+ioff,0,sim.time[i])
        for j,_ in enumerate(data.name3D):
            sh_1Dto3D.write(i+ioff,j+joff,sim.mass1Dto3D[j,i])
            sum1Dto3D[i]+=sim.mass1Dto3D[j,i]
        sh_1Dto3D.write(i+ioff,sumcol,sum1Dto3D[i])

    for i in range(numMultiTimePartial):
        sh_3Dto1D.write(i+ioff,0,sim.time[i])
        for j,_ in enumerate(data.name3D):
            sh_3Dto1D.write(i+ioff,j+joff,sim.mass3Dto1D[j,i])
            sum3Dto1D[i]+=sim.mass3Dto1D[j,i]
        sh_3Dto1D.write(i+ioff,sumcol,sum3Dto1D[i])

    return sum3Dto1D,sum1Dto3D


def write_1Ddeposition(workbook,sim):
    #sim.mass1Ddeposited[ndist,ngen,ntimes]
    mass1Ddepovergens=np.sum(sim.mass1Ddeposited,axis=1)
    sum1Ddep=np.sum(mass1Ddepovergens,axis=0)

    ClusterSum=np.zeros([len(SubtreeClusters),numMultiTimePartial])
    for j,cluster in enumerate(SubtreeClusters):
        for subtree in cluster:
            ClusterSum[j,:]+=mass1Ddepovergens[subtree-1,:numMultiTimePartial]  #Subtrees are 1-based

    sheetname='Dep1D'
    sh_dep1D=workbook.add_worksheet(sheetname)
    ioff=2
    sh_dep1D.write(0,0,'Time')
    sh_dep1D.write(0,1,'Generation')
    nextcol=0
    for i in range(numMultiTimePartial):
        for k in range(sim.ngen):
            sh_dep1D.write(i*sim.ngen+k+ioff,nextcol,sim.time[i])
            sh_dep1D.write(i*sim.ngen+k+ioff,nextcol+1,k)
    nextcol+=2
    for j,_ in enumerate(data.name3D):
        sh_dep1D.write(0,j+nextcol,data.name3D[j])
        sh_dep1D.write(1,j+nextcol,data.name1D[j])
        for i in range(numMultiTimePartial):
            for k in range(sim.ngen):
                sh_dep1D.write(i*sim.ngen+k+ioff,nextcol+j,
                        sim.mass1Ddeposited[j,k,i])

    nextcol+=len(data.name3D)+1

    sh_dep1D.write(0,nextcol,'time')
    for i in range(numMultiTimePartial):
        sh_dep1D.write(i+ioff,nextcol,sim.time[i])
    nextcol+=1
    for j,_ in enumerate(data.name3D):
        sh_dep1D.write(0,j+nextcol,f'sumgens {data.name3D[j]}')
        sh_dep1D.write(1,j+nextcol,f'sumgens {data.name1D[j]}')
        for i in range(numMultiTimePartial):
            sh_dep1D.write(i+ioff,nextcol+j,mass1Ddepovergens[j,i])

    nextcol+=len(data.name3D)+2
    sh_dep1D.write(0,nextcol,'sumgens')
    for i in range(numMultiTimePartial):
        sh_dep1D.write(i+ioff,nextcol,sum1Ddep[i])

    nextcol+=2

    clusterchart = workbook.add_chart({'type': 'scatter', 'subtype': 'straight_with_markers'})        

    sh_dep1D.write(0,nextcol,'time')
    for i in range(numMultiTimePartial):
        sh_dep1D.write(i+ioff,nextcol,sim.time[i])
    timecol=nextcol    
    nextcol+=1
    for j,_ in enumerate(SubtreeClusters):
        sh_dep1D.write(0,nextcol,SubtreeClusterNames[j])
        for i in range(numMultiTimePartial):
            sh_dep1D.write(i+ioff,nextcol,ClusterSum[j,i])
        ilast=ioff+numMultiTimePartial-1    
        clusterchart.add_series({
            'name': SubtreeClusterNames[j],
            'categories': [sheetname,ioff,timecol,ilast,timecol],
            'values': [sheetname,ioff,nextcol,ilast,nextcol],
            'line': {'color': colors()[j][1]},
            'marker': {
                'type': 'square',
                'size': 2,
                'border': {'color': colors()[j][1]},
                'fill':   {'color': colors()[j][1]},
                },
            })
        nextcol+=1

    sh_dep1D.insert_chart('J7', clusterchart)

    return sum1Ddep

    
def write_1Dsuspended(workbook,sim):
    #sim.mass1Dsuspended[ndist,ngen,ntimes]
    mass1Dsuspovergens=np.sum(sim.mass1Dsuspended,axis=1)
    sum1Dsusp=np.sum(mass1Dsuspovergens,axis=0)

    ClusterSum=np.zeros([len(SubtreeClusters),numMultiTimePartial])
    for j,cluster in enumerate(SubtreeClusters):
        for subtree in cluster:
            ClusterSum[j,:]+=mass1Dsuspovergens[subtree-1,:numMultiTimePartial]  #Subtrees are 1-based

    sheetname='Susp1D'
    sh_susp1D=workbook.add_worksheet(sheetname)
    ioff=2
    sh_susp1D.write(0,0,'Time')
    sh_susp1D.write(0,1,'Generation')
    nextcol=0
    for i in range(numMultiTimePartial):
        for k in range(sim.ngen):
            sh_susp1D.write(i*sim.ngen+k+ioff,nextcol,sim.time[i])
            sh_susp1D.write(i*sim.ngen+k+ioff,nextcol+1,k)
    nextcol+=2
    for j,_ in enumerate(data.name3D):
        sh_susp1D.write(0,j+nextcol,data.name3D[j])
        sh_susp1D.write(1,j+nextcol,data.name1D[j])
        for i in range(numMultiTimePartial):
            for k in range(sim.ngen):
                sh_susp1D.write(i*sim.ngen+k+ioff,nextcol+j,
                        sim.mass1Dsuspended[j,k,i])

    nextcol+=len(data.name3D)+1

    sh_susp1D.write(0,nextcol,'time')
    for i in range(numMultiTimePartial):
        sh_susp1D.write(i+ioff,nextcol,sim.time[i])
    nextcol+=1    
    for j,_ in enumerate(data.name3D):
        sh_susp1D.write(0,j+nextcol,f'sumgens {data.name3D[j]}')
        sh_susp1D.write(1,j+nextcol,f'sumgens {data.name1D[j]}')
        for i in range(numMultiTimePartial):
            sh_susp1D.write(i+ioff,nextcol+j,mass1Dsuspovergens[j,i])

    nextcol+=len(data.name3D)+2
    sh_susp1D.write(0,nextcol,'sumgens')
    for i in range(numMultiTimePartial):
        sh_susp1D.write(i+ioff,nextcol,sum1Dsusp[i])
    
    nextcol+=2

    clusterchart = workbook.add_chart({'type': 'scatter', 'subtype': 'straight_with_markers'})        

    sh_susp1D.write(0,nextcol,'time')
    for i in range(numMultiTimePartial):
        sh_susp1D.write(i+ioff,nextcol,sim.time[i])
    timecol=nextcol    
    nextcol+=1
    for j,_ in enumerate(SubtreeClusters):
        sh_susp1D.write(0,nextcol,SubtreeClusterNames[j])
        for i in range(numMultiTimePartial):
            sh_susp1D.write(i+ioff,nextcol,ClusterSum[j,i])
        ilast=ioff+numMultiTimePartial-1    
        clusterchart.add_series({
            'name': SubtreeClusterNames[j],
            'categories': [sheetname,ioff,timecol,ilast,timecol],
            'values': [sheetname,ioff,nextcol,ilast,nextcol],
            'line': {'color': colors()[j][1]},
            'marker': {
                'type': 'square',
                'size': 2,
                'border': {'color': colors()[j][1]},
                'fill':   {'color': colors()[j][1]},
                },
            })
        nextcol+=1

    sh_susp1D.insert_chart('J7', clusterchart)
    
    return sum1Dsusp


def write_1Ddep_by_gen(workbook,sim):
    #sim.mass1Ddeposited[ndist,ngen,ntimes]
    mass1Ddepoversubtrees=np.sum(sim.mass1Ddeposited,axis=0)

    ClusterSum=np.zeros([len(GenClusters),numMultiTimePartial])
    for j,cluster in enumerate(GenClusters):
        for gen in cluster:
            ClusterSum[j,:]+=mass1Ddepoversubtrees[gen,:numMultiTimePartial]  #gens are 0-based

    sheetname='Dep1DbyGen'
    sh_dep1D=workbook.add_worksheet(sheetname)
    ioff=1
    clusterchart = workbook.add_chart({'type': 'scatter', 'subtype': 'straight_with_markers'})        
    nextcol=0

    sh_dep1D.write(0,nextcol,'time')
    for i in range(numMultiTimePartial):
        sh_dep1D.write(i+ioff,nextcol,sim.time[i])
    timecol=nextcol    
    nextcol+=1
    for j,_ in enumerate(GenClusters):
        sh_dep1D.write(0,nextcol,GenClusterNames[j])
        for i in range(numMultiTimePartial):
            sh_dep1D.write(i+ioff,nextcol,ClusterSum[j,i])
        ilast=ioff+numMultiTimePartial-1    
        clusterchart.add_series({
            'name': GenClusterNames[j],
            'categories': [sheetname,ioff,timecol,ilast,timecol],
            'values': [sheetname,ioff,nextcol,ilast,nextcol],
            'line': {'color': colors()[j][1]},
            'marker': {
                'type': 'square',
                'size': 2,
                'border': {'color': colors()[j][1]},
                'fill':   {'color': colors()[j][1]},
                },
            })
        nextcol+=1

    sh_dep1D.insert_chart('A7', clusterchart)

def write_1Dsusp_by_gen(workbook,sim):
    #sim.mass1Ddeposited[ndist,ngen,ntimes]
    mass1Dsuspoversubtrees=np.sum(sim.mass1Dsuspended,axis=0)

    ClusterSum=np.zeros([len(GenClusters),numMultiTimePartial])
    for j,cluster in enumerate(GenClusters):
        for gen in cluster:
            ClusterSum[j,:]+=mass1Dsuspoversubtrees[gen,:numMultiTimePartial]  #gens are 0-based

    sheetname='Susp1DbyGen'
    sh_susp1D=workbook.add_worksheet(sheetname)
    ioff=1
    clusterchart = workbook.add_chart({'type': 'scatter', 'subtype': 'straight_with_markers'})        
    nextcol=0

    sh_susp1D.write(0,nextcol,'time')
    for i in range(numMultiTimePartial):
        sh_susp1D.write(i+ioff,nextcol,sim.time[i])
    timecol=nextcol    
    nextcol+=1
    for j,_ in enumerate(GenClusters):
        sh_susp1D.write(0,nextcol,GenClusterNames[j])
        for i in range(numMultiTimePartial):
            sh_susp1D.write(i+ioff,nextcol,ClusterSum[j,i])
        ilast=ioff+numMultiTimePartial-1    
        clusterchart.add_series({
            'name': GenClusterNames[j],
            'categories': [sheetname,ioff,timecol,ilast,timecol],
            'values': [sheetname,ioff,nextcol,ilast,nextcol],
            'line': {'color': colors()[j][1]},
            'marker': {
                'type': 'square',
                'size': 2,
                'border': {'color': colors()[j][1]},
                'fill':   {'color': colors()[j][1]},
                },
            })
        nextcol+=1

    sh_susp1D.insert_chart('A7', clusterchart)
    
def write_1Ddep_by_subtree_and_gen(workbook,sim):
    #sim.mass1Ddeposited[ndist,ngen,ntimes]

    sheetname='Dep1DbySubtree+Gen'

    CoClusterSum=np.zeros([len(SubtreeGenCoClusters),numMultiTimePartial])
    for ind,cocluster in enumerate(SubtreeGenCoClusters):
        for subtree in cocluster[0]:
            for gen in cocluster[1]:
                # Subtrees are 1-based; gens are 0-based
                CoClusterSum[ind,:]+=sim.mass1Ddeposited[subtree-1,gen,:numMultiTimePartial]

    sh_dep1D=workbook.add_worksheet(sheetname)
    ioff=1
    clusterchart = workbook.add_chart({'type': 'scatter', 'subtype': 'straight_with_markers'})        
    nextcol=0

    sh_dep1D.write(0,nextcol,'time')
    for i in range(numMultiTimePartial):
        sh_dep1D.write(i+ioff,nextcol,sim.time[i])
    timecol=nextcol    
    nextcol+=1
    for ind,_ in enumerate(SubtreeGenCoClusters):
        sh_dep1D.write(0,nextcol,SubtreeGenCoClusterNames[ind])
        for i in range(numMultiTimePartial):
            sh_dep1D.write(i+ioff,nextcol,CoClusterSum[ind,i])
        ilast=ioff+numMultiTimePartial-1    
        clusterchart.add_series({
            'name': SubtreeGenCoClusterNames[ind],
            'categories': [sheetname,ioff,timecol,ilast,timecol],
            'values': [sheetname,ioff,nextcol,ilast,nextcol],
            'line': {'color': colors()[ind][1]},
            'marker': {
                'type': 'square',
                'size': 2,
                'border': {'color': colors()[ind][1]},
                'fill':   {'color': colors()[ind][1]},
                },
            })
        nextcol+=1

    sh_dep1D.insert_chart('A7', clusterchart)

def write_1Dsusp_by_subtree_and_gen(workbook,sim):
    #sim.mass1Dsuspended[ndist,ngen,ntimes]

    sheetname='Susp1DbySubtree+Gen'

    CoClusterSum=np.zeros([len(SubtreeGenCoClusters),numMultiTimePartial])
    for ind,cocluster in enumerate(SubtreeGenCoClusters):
        for subtree in cocluster[0]:
            for gen in cocluster[1]:
                # Subtrees are 1-based; gens are 0-based
                CoClusterSum[ind,:]+=sim.mass1Dsuspended[subtree-1,gen,:numMultiTimePartial]

    sh_susp1D=workbook.add_worksheet(sheetname)
    ioff=1
    clusterchart = workbook.add_chart({'type': 'scatter', 'subtype': 'straight_with_markers'})        
    nextcol=0

    sh_susp1D.write(0,nextcol,'time')
    for i in range(numMultiTimePartial):
        sh_susp1D.write(i+ioff,nextcol,sim.time[i])
    timecol=nextcol    
    nextcol+=1
    for ind,_ in enumerate(SubtreeGenCoClusters):
        sh_susp1D.write(0,nextcol,SubtreeGenCoClusterNames[ind])
        for i in range(numMultiTimePartial):
            sh_susp1D.write(i+ioff,nextcol,CoClusterSum[ind,i])
        ilast=ioff+numMultiTimePartial-1    
        clusterchart.add_series({
            'name': SubtreeGenCoClusterNames[ind],
            'categories': [sheetname,ioff,timecol,ilast,timecol],
            'values': [sheetname,ioff,nextcol,ilast,nextcol],
            'line': {'color': colors()[ind][1]},
            'marker': {
                'type': 'square',
                'size': 2,
                'border': {'color': colors()[ind][1]},
                'fill':   {'color': colors()[ind][1]},
                },
            })
        nextcol+=1

    sh_susp1D.insert_chart('A7', clusterchart)

def write_per_breath(worksheet,sim,sum3Dto1D,sum1Dto3D):
    exind=[indexwithtol(sim.inhales[0][0],sim.time)] #Zeroth breath begins at this index (probably index zero).
    for tstart,tend in sim.exhales:
        exind+=[indexwithtol(tend,sim.time)] 

    inhind=[]
    for tstart,tend in sim.inhales:
        inhind+=[indexwithtol(tend,sim.time)]

    worksheet.write(0,0,'Accumulation Over One Complete Breath at End Exhale')
    nextcol=0
    worksheet.write(1,nextcol,'Time')
    nextcol+=1
    nextcol+=1
    worksheet.write(1,nextcol,'\u0394 Inhale 3D') #\u0394 is Capital Delta character
    nextcol+=1
    if sim.nDepPatch>1:
        for j,patch in enumerate(data.depPatch):
            worksheet.write(1,nextcol,f'\u0394 dep {patch}')
            nextcol+=1
    worksheet.write(1,nextcol,'\u0394 Total Dep 3D')
    nextcol+=1
    worksheet.write(1,nextcol,'\u0394 3D->1D')
    nextcol+=1
    nextcol+=1
    worksheet.write(1,nextcol,'\u0394 1D->3D')
    nextcol+=1
    worksheet.write(1,nextcol,'\u0394 1D system mass')
    nextcol+=1
    worksheet.write(1,nextcol,'\u0394 Dep 1D')
    nextcol+=1
    nextcol+=1
    worksheet.write(1,nextcol,'\u0394 Exhale 3D')
    nextcol+=1
            
    net1Dmass=sum3Dto1D[:numMultiTimePartial]-sum1Dto3D[:numMultiTimePartial]
    mass1Ddepovergens=np.sum(sim.mass1Ddeposited,axis=1)
    sum1Ddep=np.sum(mass1Ddepovergens,axis=0)
    mass1Dsuspovergens=np.sum(sim.mass1Dsuspended,axis=1)
    sum1Dsusp=np.sum(mass1Dsuspovergens,axis=0)
    totMass3Ddeposited=np.sum(sim.mass3Ddeposited,axis=0)

    ioff=2
    for i,indx in enumerate(exind):
        if i==0:  
            continue
        else:
            prev=exind[i-1]
        nextcol=0
        worksheet.write(ioff+i-1,nextcol,sim.time[indx])
        nextcol+=1
        nextcol+=1
        if sim.time[indx]<=sim.multiTimeCompleted+TOL:
            worksheet.write(ioff+i-1,nextcol,sim.massInhaled[indx]-sim.massInhaled[prev])
        nextcol+=1
        if sim.nDepPatch>1:
            for j,patch in enumerate(data.depPatch):
                if sim.time[indx]<=sim.multiTimeCompleted+TOL:
                    worksheet.write(ioff+i-1,nextcol,sim.mass3Ddeposited[j,indx]-
                            sim.mass3Ddeposited[j,prev])       
                nextcol+=1
        if sim.time[indx]<=sim.multiTimeCompleted+TOL:
            worksheet.write(ioff+i-1,nextcol,totMass3Ddeposited[indx]-totMass3Ddeposited[prev])
        nextcol+=1
        if sim.time[indx]<=sim.multiTimeCompleted+TOL:
            worksheet.write(ioff+i-1,nextcol,sum3Dto1D[indx]-sum3Dto1D[prev])
        nextcol+=1
        nextcol+=1
        worksheet.write(ioff+i-1,nextcol,sum1Dto3D[indx]-sum1Dto3D[prev])
        nextcol+=1
        worksheet.write(ioff+i-1,nextcol,net1Dmass[indx]-net1Dmass[prev])
        nextcol+=1
        worksheet.write(ioff+i-1,nextcol,sum1Ddep[indx]-sum1Ddep[prev])
        nextcol+=1
        nextcol+=1
        if sim.time[indx]<=sim.multiTimeCompleted+TOL:
            worksheet.write(ioff+i-1,nextcol,sim.massExhaled[indx]-sim.massExhaled[prev])
        nextcol+=1
                    
    ioff=len(exind)+5
    worksheet.write(ioff,0,'Susp End Inhale')
    worksheet.write(ioff+1,0,'Time')
    worksheet.write(ioff+1,1,'Susp 3D')
    worksheet.write(ioff+1,2,'Susp 1D')
    for i,indx in enumerate(inhind):
        worksheet.write(ioff+i+2,0,sim.time[indx])
        worksheet.write(ioff+i+2,1,sim.mass3Dsuspended[indx])
        worksheet.write(ioff+i+2,2,sum1Dsusp[indx])

    worksheet.write(ioff,5,'Susp End Exhale')
    worksheet.write(ioff+1,5,'Time')
    worksheet.write(ioff+1,6,'Susp 3D')
    worksheet.write(ioff+1,7,'Susp 1D')
    for i,indx in enumerate(exind[1:]):
        worksheet.write(ioff+i+2,5,sim.time[indx])
        if sim.time[indx]<=sim.multiTimeCompleted+TOL:
            worksheet.write(ioff+i+2,6,sim.mass3Dsuspended[indx])
        worksheet.write(ioff+i+2,7,sum1Dsusp[indx])


#colors[i][1] are color strings recognized by xlsxwriter module
#colors[i][0] are RGB triplets.  In theory could randomly find 
#points in RGB spaced for additional colors (after checking
#sufficient distance to previously accepted colors)
#Those would be specified as e.g. '#FF0000'
def colors():
    colors=[]
    colors+=[[(0,0,0), 'black']]
    colors+=[[(0,0,255), 'blue']]
    colors+=[[(128,0,0), 'brown']]
    colors+=[[(128,128,128), 'gray']]
    colors+=[[(0,255,255), 'cyan']]
    colors+=[[(0,128,0), 'green']]
    colors+=[[(0,255,0), 'lime']]
    colors+=[[(255,0,255), 'magenta']]
    colors+=[[(0,0,128), 'navy']]
    colors+=[[(255,102,0), 'orange']]
    colors+=[[(255,0,255), 'pink']]
    colors+=[[(128,0,128), 'purple']]
    colors+=[[(255,0,0), 'red']]
    colors+=[[(192,192,192), 'silver']]
#    colors+=[[(255,255,255), 'white']]  #white not good for charts
    colors+=[[(255,255,0), 'yellow']]
    return colors

TOL=1.e-8
small=lambda arr : np.amax(np.abs(arr)) < TOL
def indexwithtol(val,arr):
    a=(np.abs(arr-val)).argmin()
    assert small(arr[a]-val), "cant find time in array"
    return a

if __name__ == "__main__":
    main(sys.argv[1:])
    
