"""post3D.py--- this module retrieves output from 3D code"""

import os
import sys
import glob
import simclass
import numpy as np
from data import name1D,name3D,depPatch
from comm3D import readinjection
import yaml

TOL=1.e-8
small=lambda arr : np.amax(np.abs(arr)) < TOL

def post3Dinhale(sim: simclass.SimClass): 
    """FOAMSPECIFIC-- read 3D output after inhalation and write to sim object"""

    #tstart3D is beginning of current inbreath.  tstart specified in data.py may
    #come after start of inhalation (due to restart), but we are analyzing entire inbreath here.

    tstart3D=sim.inhales[sim.currbreath][0]
    tend3D=min(sim.tend,sim.inhales[sim.currbreath][1])

    sim.multiTimeCompleted=tend3D
    newtimes=round((tend3D-tstart3D)/sim.dtio) #The 3D start time isnt counted
    tendexh=sim.exhales[sim.currbreath][1]
    sim.multiTimePartial=tendexh
    fulltimes=round((tendexh-tstart3D)/sim.dtio) #times of anticipated flows
    readinjection(sim) #Loads sim.massInhaled
    a=np.abs(tstart3D+sim.dtio-sim.time).argmin()
    assert small(np.abs(tstart3D+sim.dtio-sim.time[a])), "sim not ready for new times"

    writedir=f'multiResults/{sim.currbreath}_inh'
    if os.system(f'mkdir -p {writedir}'):
        sys.exit(f'post3Dinhale: Cannot create {writedir}')

    for ind,out3D in enumerate(name3D):
        path=f'postProcessing/flowRatePatch(name={out3D},phi.air,'\
              f'executeControl=writeTime,writeControl=writeTime)'
        simupdate(sim.recflowrate3D[ind,a:a+newtimes],sim.time[a:a+newtimes],path)      

        path=f'postProcessing/flowRatePatch(name={out3D},kinematicCloud:massEscape,'\
              f'executeControl=writeTime,writeControl=writeTime)'
        simupdate(sim.mass3Dto1D[ind,a:a+newtimes],sim.time[a:a+newtimes],path)      
                
        writefile=f'{writedir}/airvol_pmass_{name1D[ind]}.csv'
# Flow integrated volume. Volumes must be maxxed with zero, since
# MPPD cannot tolerate the tiniest negative flow volume.
        write_csv_3D1D(writefile,sim.time[a-1:a+fulltimes]-sim.time[a-1],
                np.maximum(0.,
                sim.specairvols[ind,a-1:a+fulltimes]-sim.specairvols[ind,a-1]),
                sim.mass3Dto1D[ind,a-1:a+newtimes]-sim.mass3Dto1D[ind,a-1])

# Read 3D deposition        
    for ind,patch in enumerate(depPatch):
        path=f'postProcessing/flowRatePatch(name={patch},kinematicCloud:massStick,'\
           f'executeControl=writeTime,writeControl=writeTime)'
        simupdate(sim.mass3Ddeposited[ind,a:a+newtimes],sim.time[a:a+newtimes],path)      

# Copy constant values of fields during inhale phase
    sim.massExhaled[a:a+newtimes]=sim.massExhaled[a-1]
    for ind,_ in enumerate(name3D):
        sim.mass1Dto3D[ind,a:a+newtimes]=sim.mass1Dto3D[ind,a-1]

# Copy constant values of fields during coming exhale phase
    sim.massInhaled[a+newtimes:a+fulltimes]=sim.massInhaled[a-1+newtimes]
    for ind,_ in enumerate(name3D):
        sim.mass3Dto1D[ind,a+newtimes:a+fulltimes]=sim.mass3Dto1D[ind,a-1+newtimes]

# Call 1D MPPD code since need mass1Dto3D values for the following.
    import call1D
    call1D.call1D(sim)

# Compute 3D suspended mass
    sim.mass3Dsuspended[a:a+newtimes]=(
            + sim.massInhaled[a:a+newtimes] 
            - sim.massExhaled[a:a+newtimes] #### check defined
            + np.sum(sim.mass1Dto3D, axis=0)[a:a+newtimes]  ###check defined
            - np.sum(sim.mass3Dto1D, axis=0)[a:a+newtimes]
            - np.sum(sim.mass3Ddeposited, axis=0)[a:a+newtimes] )

# Dump sim object to file.  Put in a time-named directory.
    file_simdump=f'{writedir}/sim.yaml'
    with open(file_simdump,'w') as f:
        yaml.dump(sim,f)
# Dump copy of 3D log file (typically 'Lung.out') to same directory 
    print(f'\nCopying {sim.output3Dfile} to {writedir}\n', \
            flush=True) 
    if os.system(f'/bin/cp -p {sim.output3Dfile} {writedir}'):
        print(f'Warning: post3Dinhale: Cannot make copy of {sim.output3Dfile}',
                flush=True)
    
def post3Dexhale(sim: simclass.SimClass):
    """FOAMSPECIFIC-- read 3D output after exhalation and write to sim object"""

    #tstart3D is beginning of current outbreath.  tstart specified in data.py may
    #come after start of exhalation (due to restart), but we are analyzing entire outbreath here.

    tstart3D=sim.exhales[sim.currbreath][0]
    tend3D=min(sim.tend,sim.exhales[sim.currbreath][1])

    sim.multiTimeCompleted=tend3D
    sim.multiTimePartial=tend3D
    newtimes=round((tend3D-tstart3D)/sim.dtio) #The 3D start time isnt counted

    #readinjection(sim) #readinjection could get injection from Lung.out.  But we have 
    # sim.mass1Dto3D already computed by MPPD.  

    a=np.abs(tstart3D+sim.dtio-sim.time).argmin()
    assert small(np.abs(tstart3D+sim.dtio-sim.time[a])), "sim not ready for new times"
    for ind,out3D in enumerate(name3D):
        path=f'postProcessing/flowRatePatch(name={out3D},phi.air,'\
              f'executeControl=writeTime,writeControl=writeTime)'
        simupdate(sim.recflowrate3D[ind,a:a+newtimes],sim.time[a:a+newtimes],path)      

    path='postProcessing/flowRatePatch(name=inlet,kinematicCloud:massEscape,'\
          f'executeControl=writeTime,writeControl=writeTime)'
    simupdate(sim.massExhaled[a:a+newtimes],sim.time[a:a+newtimes],path)      

# Read 3D deposition        
    for ind,patch in enumerate(depPatch):
        path=f'postProcessing/flowRatePatch(name={patch},kinematicCloud:massStick,'\
           f'executeControl=writeTime,writeControl=writeTime)'
        simupdate(sim.mass3Ddeposited[ind,a:a+newtimes],sim.time[a:a+newtimes],path)      

# Copy constant values of fields during exhale phase
    sim.massInhaled[a:a+newtimes]=sim.massInhaled[a-1]
    for ind,_ in enumerate(name3D):
        sim.mass3Dto1D[ind,a:a+newtimes]=sim.mass3Dto1D[ind,a-1]

# Compute 3D suspended mass 
    sim.mass3Dsuspended[a:a+newtimes]=(
            + sim.massInhaled[a:a+newtimes] ##### check defined
            - sim.massExhaled[a:a+newtimes] 
            + np.sum(sim.mass1Dto3D, axis=0)[a:a+newtimes]
            - np.sum(sim.mass3Dto1D, axis=0)[a:a+newtimes]  ##### check defined
            - np.sum(sim.mass3Ddeposited, axis=0)[a:a+newtimes] )

# Dump sim object to file
    writedir=f'multiResults/{sim.currbreath}_exh'
    if os.system(f'mkdir -p {writedir}'):
        sys.exit(f'post3Dexhale: Cannot create {writedir}')
    file_simdump=f'{writedir}/sim.yaml'
    with open(file_simdump,'w') as f:
        yaml.dump(sim,f)
# Dump copy of 3D log file (typically 'Lung.out') to same directory 
    print(f'\nCopying {sim.output3Dfile} to {writedir}\n', \
            flush=True) 
    if os.system(f'/bin/cp -p {sim.output3Dfile} {writedir}'):
        sys.exit(f'post3Dexhale: Cannot make copy of {sim.output3Dfile}')
    
def read_file_OFpostProcess(file):
    """FOAMSPECIFIC-- read 3D field quantities from FOAM postProcessing directory"""
    time=[]
    field=[]
    with open(file,'r') as f:
        fcontents=f.readlines()
    for line in fcontents:
        splline=line.split()
        if len(splline)==2:
            time.append(float(splline[0]))
            field.append(float(splline[1]))
    return np.array(time),np.array(field) 

def write_csv_3D1D(writefile,time,volume,pmass):
    """Write csv file of time, continuum volume, particle mass for use by MPPD"""     
    import csv
    with open(writefile,'w') as f:
        writer=csv.writer(f)
        for i in range(len(pmass)):
            writer.writerow((time[i],volume[i],pmass[i]))
        for i in range(len(pmass),len(time)):
            writer.writerow((time[i],volume[i],pmass[-1]))

def simupdate(simarray,simtime,path):
    #Patch together values for times in simtime.  Look at files in reverse chrono order.
    #(Older files are obsolete in cases of restarts.)
    ttargsimind=len(simtime)-1
    print(f'processing directory: {path}',flush=True)
    sortedfiles=sorted(glob.glob(f'{path}/*/surfaceFieldValue*.dat'),
            key=os.path.getmtime, reverse=True)
    for cand in sortedfiles:
        print(f'processing: {cand}',flush=True)
        try:
            time,val=read_file_OFpostProcess(cand)
            indtarg=(np.abs(simtime[ttargsimind]-time)).argmin()
        except:
            continue
        if small(np.abs(simtime[ttargsimind]-time[indtarg])): # File must have LAST time
            if small(np.abs(simtime[0]-time[0])): # File provides ALL times
                copy_and_verify(simarray[:ttargsimind+1],simtime[:ttargsimind+1],val,time)
                return
            indstart=(np.abs(simtime-time[0])).argmin()  # Beginning time that file has
            if not small(np.abs(simtime[indstart]-time[0])):
                sys.exit(f'Invalid time at beginning of file {cand}')
            copy_and_verify(simarray[indstart:ttargsimind+1],simtime[indstart:ttargsimind+1],
                        val,time)
            ttargsimind=indstart-1 # Look for remaining times in a previously written file
    sys.exit('Could not gather postProcessing data for simupdate')

def copy_and_verify(sval,stime,val,time):
    for ind,_ in enumerate(stime):
        if not small(np.abs(stime[ind]-time[ind])):
            sys.exit('Times do not match up in values copy')
        sval[ind]=val[ind]
    return

