"""pre3D.py--- this module sets parameters for 3D code inhale or exhale"""

import simclass
import glob
import os
from data import injectors, shortinjectors
from comm3D import editentry,genparticle_dmdt_inlet,genparticle_dmdt_outlets

def pre3Dinhale(sim: simclass.SimClass):
    """FOAMSPECIFIC-- set parameters to be read by 3D code for next inhale"""
    genparticle_dmdt_inlet(sim) # time offsets tstart off of 'absolute' inhale profile
    tstart3D=max(sim.tstart,sim.inhales[sim.currbreath][0])
    tend3D=min(sim.tend,sim.inhales[sim.currbreath][1])
    editentry("^startTime",tstart3D,'system/controlDict')
    editentry("^endTime",tend3D,'system/controlDict')
    editentry("^writeInterval",sim.dtio,'system/controlDict')
    editentry("^maxCo",sim.params3D["maxCoFlow"],'system/controlDict')
    editentry("^maxDeltaT",sim.params3D["maxDeltaT"],'system/controlDict')
 
    SOI=sim.inhales[sim.currbreath][0] #Time of start of injection
    duration=sim.inhales[sim.currbreath][1]-SOI

    editentry("rho0",sim.particleDensity,"constant/kinematicCloudProperties")
    editentry("maxCo",sim.params3D["maxCoParticle"],"constant/kinematicCloudProperties")
    editentry("#include",'"inletInjections.H"',"constant/kinematicCloudProperties")

    # Go thru all U.air processor files and change any flowRateInletVelocity boundary condition
    # flowRateOutletVelocity which is the correct BC for 3D/1D interfaces during inhale.
    strtime=str(round(tstart3D, 6 ))
    strtime=strtime.rstrip('0').rstrip('.')
    processorfiles=glob.glob(f'processor*/{strtime}/U.air')
    for proc in processorfiles:
        com=f"sed -i '/flowRateInletVelocity;/ c\ \t\t type\t \t flowRateOutletVelocity;' {proc}"
        if os.system(com):
            sys.exit(f'pre3D: Problem with specifying U.air boundary condition')
        
    with open("constant/inletInjections.H","w") as f:
        massTotal=sim.inhaleMasses[sim.currbreath] 
        parcelsPerSecond=sim.params3D["inhaleParcelsPerSecond"]
        f.write(f'{injectors[-1]}\n') #'Last' injector is inletInjection_inlet.
        f.write('{\n')
        f.write('type patchInjection;\n')
        f.write(f'SOI {SOI};\n')
        f.write('parcelBasisType mass;\n')
        f.write(f'duration {duration};\n')
        f.write('U0 (0 0 0);\n')
        f.write('sizeDistribution\n')
        f.write('{\n')
        f.write('    type fixedValue;\n')
        f.write('    fixedValueDistribution\n')
        f.write('    {\n')
        f.write(f'        value {sim.particleDiameter};\n')
        f.write('    }\n')
        f.write('}\n')
        f.write(f'patch\t{shortinjectors[-1]};\n') 
        f.write(f'patchName\t{shortinjectors[-1]};\n') 
        f.write(f'parcelsPerSecond\t{parcelsPerSecond};\n')
        f.write(f'massTotal\t{massTotal};\n')
        f.write('flowRateProfile tableFile;\n')
        f.write(f'file "particle_dmdt_{shortinjectors[-1]}.txt";\n')
        f.write('outOfBounds\tclamp;\n')
        f.write('}\n\n')

def pre3Dexhale(sim: simclass.SimClass):
    """FOAMSPECIFIC-- set parameters to be read by 3D code for next exhale"""

    genparticle_dmdt_outlets(sim)
    tstart3D=max(sim.tstart,sim.exhales[sim.currbreath][0])
    tend3D=min(sim.tend,sim.exhales[sim.currbreath][1])
    editentry("^startTime",tstart3D,'system/controlDict')
    editentry("^endTime",tend3D,'system/controlDict')
    editentry("^writeInterval",sim.dtio,'system/controlDict')
    editentry("^maxCo",sim.params3D["maxCoFlow"],'system/controlDict')
    editentry("^maxDeltaT",sim.params3D["maxDeltaT"],'system/controlDict')

    SOI=sim.exhales[sim.currbreath][0] #Time of start of injection
    duration=sim.exhales[sim.currbreath][1]-SOI

    editentry("rho0",sim.particleDensity,"constant/kinematicCloudProperties")
    editentry("maxCo",sim.params3D["maxCoParticle"],"constant/kinematicCloudProperties")
    editentry("#include",'"outletInjections.H"',"constant/kinematicCloudProperties")

    # Go thru all U.air processor files and change any flowRateOutletVelocity boundary condition
    # flowRateInletVelocity which is the correct BC for 3D/1D interfaces during exhale.
    strtime=str(round(tstart3D, 6 ))
    strtime=strtime.rstrip('0').rstrip('.')
    processorfiles=glob.glob(f'processor*/{strtime}/U.air')
    for proc in processorfiles:
        com=f"sed -i '/flowRateOutletVelocity;/ c\ \t\t type\t \t flowRateInletVelocity;' {proc}"
        if os.system(com):
            sys.exit(f'pre3D: Problem with specifying U.air boundary condition')

    with open("constant/outletInjections.H","w") as f:
        for ind,_ in enumerate(injectors):
            if ind<len(injectors)-1:  #Last injector len(injectors)-1 is for inlet
                tstart3D=sim.inhales[sim.currbreath][0]
                tendinh3D=sim.inhales[sim.currbreath][1]
                tendexh3D=sim.exhales[sim.currbreath][1]
                ninhtimes=round((tendinh3D-tstart3D)/sim.dtio)
                ntimes=round((tendexh3D-tstart3D)/sim.dtio)
                noff=round(tstart3D/sim.dtio)
                exh=sim.mass1Dto3D[ind,noff:]
                massTotal=exh[ntimes]-exh[ninhtimes] 
                parcelsPerSecond=sim.parcelsPerKilo*massTotal/(tendexh3D-tendinh3D)
                f.write(f'{injectors[ind]}\n')
                f.write('{\n')
                f.write('type patchInjection;\n')
                f.write(f'SOI {SOI};\n')
                f.write('parcelBasisType mass;\n')
                f.write(f'duration {duration};\n')
                f.write('U0 (0 0 0);\n')
                f.write('sizeDistribution\n')
                f.write('{\n')
                f.write('    type fixedValue;\n')
                f.write('    fixedValueDistribution\n')
                f.write('    {\n')
                f.write(f'        value {sim.particleDiameter};\n')
                f.write('    }\n')
                f.write('}\n')
                f.write(f'patch\t{shortinjectors[ind]};\n') 
                f.write(f'patchName\t{shortinjectors[ind]};\n') 
                f.write(f'parcelsPerSecond\t{parcelsPerSecond};\n')
                f.write(f'massTotal\t{massTotal};\n')
                f.write('flowRateProfile tableFile;\n')
                f.write(f'file "particle_dmdt_{shortinjectors[ind]}.txt";\n')
                f.write('outOfBounds\tclamp;\n')
                f.write('}\n\n')

